@@ -1,4 +1,5 @@
 /*
+ *  SVGTips device, (C) 2008 Tony Plate, based on
  *  SVG device, (C) 2002 T Jake Luciani, Based on PicTex device, for
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
@@ -109,6 +110,83 @@ typedef struct {
  * use data from the first vector, some special characters seem to draw
  * on data from the others.
  */
+
+/* This charwidth works for font-family="Arial" in FireFox under Ubuntu */
+
+static double charwidth[4][128] = {
+    {
+0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056,
+0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056,
+0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056,
+0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056,
+0.42, 0.42, 0.504, 0.7056, 0.7056, 1.1928, 0.8904, 0.3024,
+0.42, 0.42, 0.504, 0.7896, 0.42, 0.42, 0.42, 0.42,
+0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056,
+0.7056, 0.7056, 0.42, 0.42, 0.7896, 0.7896, 0.7896, 0.7056,
+1.3944, 0.8904, 0.8904, 0.9912, 0.9912, 0.8904, 0.7896, 0.9912,
+0.9912, 0.42, 0.7056, 0.8904, 0.7056, 1.092, 0.9912, 0.9912,
+0.8904, 0.9912, 0.9912, 0.8904, 0.7896, 0.9912, 0.8904, 1.2936,
+0.8904, 0.8904, 0.7896, 0.42, 0.42, 0.42, 0.6048, 0.7056,
+0.42, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.42, 0.7056,
+0.7056, 0.3024, 0.3024, 0.7056, 0.3024, 1.092, 0.7056, 0.7056,
+0.7056, 0.7056, 0.42, 0.7056, 0.42, 0.7056, 0.7056, 0.9912,
+0.7056, 0.7056, 0.7056, 0.42, 0.3024, 0.42, 0.7896, 1.2936
+    },{
+0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056,
+0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056,
+0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056,
+0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056,
+0.42, 0.42, 0.504, 0.7056, 0.7056, 1.1928, 0.8904, 0.3024,
+0.42, 0.42, 0.504, 0.7896, 0.42, 0.42, 0.42, 0.42,
+0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056,
+0.7056, 0.7056, 0.42, 0.42, 0.7896, 0.7896, 0.7896, 0.7056,
+1.3944, 0.8904, 0.8904, 0.9912, 0.9912, 0.8904, 0.7896, 0.9912,
+0.9912, 0.42, 0.7056, 0.8904, 0.7056, 1.092, 0.9912, 0.9912,
+0.8904, 0.9912, 0.9912, 0.8904, 0.7896, 0.9912, 0.8904, 1.2936,
+0.8904, 0.8904, 0.7896, 0.42, 0.42, 0.42, 0.6048, 0.7056,
+0.42, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.42, 0.7056,
+0.7056, 0.3024, 0.3024, 0.7056, 0.3024, 1.092, 0.7056, 0.7056,
+0.7056, 0.7056, 0.42, 0.7056, 0.42, 0.7056, 0.7056, 0.9912,
+0.7056, 0.7056, 0.7056, 0.42, 0.3024, 0.42, 0.7896, 1.2936
+    },{
+0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056,
+0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056,
+0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056,
+0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056,
+0.42, 0.42, 0.504, 0.7056, 0.7056, 1.1928, 0.8904, 0.3024,
+0.42, 0.42, 0.504, 0.7896, 0.42, 0.42, 0.42, 0.42,
+0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056,
+0.7056, 0.7056, 0.42, 0.42, 0.7896, 0.7896, 0.7896, 0.7056,
+1.3944, 0.8904, 0.8904, 0.9912, 0.9912, 0.8904, 0.7896, 0.9912,
+0.9912, 0.42, 0.7056, 0.8904, 0.7056, 1.092, 0.9912, 0.9912,
+0.8904, 0.9912, 0.9912, 0.8904, 0.7896, 0.9912, 0.8904, 1.2936,
+0.8904, 0.8904, 0.7896, 0.42, 0.42, 0.42, 0.6048, 0.7056,
+0.42, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.42, 0.7056,
+0.7056, 0.3024, 0.3024, 0.7056, 0.3024, 1.092, 0.7056, 0.7056,
+0.7056, 0.7056, 0.42, 0.7056, 0.42, 0.7056, 0.7056, 0.9912,
+0.7056, 0.7056, 0.7056, 0.42, 0.3024, 0.42, 0.7896, 1.2936
+    },{
+0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056,
+0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056,
+0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056,
+0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056,
+0.42, 0.42, 0.504, 0.7056, 0.7056, 1.1928, 0.8904, 0.3024,
+0.42, 0.42, 0.504, 0.7896, 0.42, 0.42, 0.42, 0.42,
+0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056,
+0.7056, 0.7056, 0.42, 0.42, 0.7896, 0.7896, 0.7896, 0.7056,
+1.3944, 0.8904, 0.8904, 0.9912, 0.9912, 0.8904, 0.7896, 0.9912,
+0.9912, 0.42, 0.7056, 0.8904, 0.7056, 1.092, 0.9912, 0.9912,
+0.8904, 0.9912, 0.9912, 0.8904, 0.7896, 0.9912, 0.8904, 1.2936,
+0.8904, 0.8904, 0.7896, 0.42, 0.42, 0.42, 0.6048, 0.7056,
+0.42, 0.7056, 0.7056, 0.7056, 0.7056, 0.7056, 0.42, 0.7056,
+0.7056, 0.3024, 0.3024, 0.7056, 0.3024, 1.092, 0.7056, 0.7056,
+0.7056, 0.7056, 0.42, 0.7056, 0.42, 0.7056, 0.7056, 0.9912,
+0.7056, 0.7056, 0.7056, 0.42, 0.3024, 0.42, 0.7896, 1.2936
+    }
+};
+
+/* This charwidth works for font-family="sans-serif" in FireFox under Ubuntu */
+/*
 static double charwidth[4][128] = {
     {
 0.7896000, 0.7896000, 0.7896000, 0.7896000, 0.7896000, 0.7896000, 0.7896000,
@@ -195,6 +273,95 @@ static double charwidth[4][128] = {
 1.0920000, 0.7896000
     }
 };
+*/
+
+/*************
+ * charwidth table from RSvgDevice
+static double charwidth[4][128] = {
+    {
+	0.5416690, 0.8333360, 0.7777810, 0.6111145, 0.6666690, 0.7083380, 0.7222240,
+	0.7777810, 0.7222240, 0.7777810, 0.7222240, 0.5833360, 0.5361130, 0.5361130,
+	0.8138910, 0.8138910, 0.2388900, 0.2666680, 0.5000020, 0.5000020, 0.5000020,
+	0.5000020, 0.5000020, 0.6666700, 0.4444460, 0.4805580, 0.7222240, 0.7777810,
+	0.5000020, 0.8611145, 0.9722260, 0.7777810, 0.2388900, 0.3194460, 0.5000020,
+	0.8333360, 0.5000020, 0.8333360, 0.7583360, 0.2777790, 0.3888900, 0.3888900,
+	0.5000020, 0.7777810, 0.2777790, 0.3333340, 0.2777790, 0.5000020, 0.5000020,
+	0.5000020, 0.5000020, 0.5000020, 0.5000020, 0.5000020, 0.5000020, 0.5000020,
+	0.5000020, 0.5000020, 0.2777790, 0.2777790, 0.3194460, 0.7777810, 0.4722240,
+	0.4722240, 0.6666690, 0.6666700, 0.6666700, 0.6388910, 0.7222260, 0.5972240,
+	0.5694475, 0.6666690, 0.7083380, 0.2777810, 0.4722240, 0.6944480, 0.5416690,
+	0.8750050, 0.7083380, 0.7361130, 0.6388910, 0.7361130, 0.6458360, 0.5555570,
+	0.6805570, 0.6875050, 0.6666700, 0.9444480, 0.6666700, 0.6666700, 0.6111130,
+	0.2888900, 0.5000020, 0.2888900, 0.5000020, 0.2777790, 0.2777790, 0.4805570,
+	0.5166680, 0.4444460, 0.5166680, 0.4444460, 0.3055570, 0.5000020, 0.5166680,
+	0.2388900, 0.2666680, 0.4888920, 0.2388900, 0.7944470, 0.5166680, 0.5000020,
+	0.5166680, 0.5166680, 0.3416690, 0.3833340, 0.3611120, 0.5166680, 0.4611130,
+	0.6833360, 0.4611130, 0.4611130, 0.4347230, 0.5000020, 1.0000030, 0.5000020,
+	0.5000020, 0.5000020
+    },
+    {
+	0.5805590, 0.9166720, 0.8555600, 0.6722260, 0.7333370, 0.7944490, 0.7944490,
+	0.8555600, 0.7944490, 0.8555600, 0.7944490, 0.6416700, 0.5861150, 0.5861150,
+	0.8916720, 0.8916720, 0.2555570, 0.2861130, 0.5500030, 0.5500030, 0.5500030,
+	0.5500030, 0.5500030, 0.7333370, 0.4888920, 0.5652800, 0.7944490, 0.8555600,
+	0.5500030, 0.9472275, 1.0694500, 0.8555600, 0.2555570, 0.3666690, 0.5583360,
+	0.9166720, 0.5500030, 1.0291190, 0.8305610, 0.3055570, 0.4277800, 0.4277800,
+	0.5500030, 0.8555600, 0.3055570, 0.3666690, 0.3055570, 0.5500030, 0.5500030,
+	0.5500030, 0.5500030, 0.5500030, 0.5500030, 0.5500030, 0.5500030, 0.5500030,
+	0.5500030, 0.5500030, 0.3055570, 0.3055570, 0.3666690, 0.8555600, 0.5194470,
+	0.5194470, 0.7333370, 0.7333370, 0.7333370, 0.7027820, 0.7944490, 0.6416700,
+	0.6111145, 0.7333370, 0.7944490, 0.3305570, 0.5194470, 0.7638930, 0.5805590,
+	0.9777830, 0.7944490, 0.7944490, 0.7027820, 0.7944490, 0.7027820, 0.6111145,
+	0.7333370, 0.7638930, 0.7333370, 1.0388950, 0.7333370, 0.7333370, 0.6722260,
+	0.3430580, 0.5583360, 0.3430580, 0.5500030, 0.3055570, 0.3055570, 0.5250030,
+	0.5611140, 0.4888920, 0.5611140, 0.5111140, 0.3361130, 0.5500030, 0.5611140,
+	0.2555570, 0.2861130, 0.5305590, 0.2555570, 0.8666720, 0.5611140, 0.5500030,
+	0.5611140, 0.5611140, 0.3722250, 0.4216690, 0.4041690, 0.5611140, 0.5000030,
+	0.7444490, 0.5000030, 0.5000030, 0.4763920, 0.5500030, 1.1000060, 0.5500030,
+	0.5500030, 0.550003 },
+    {
+	0.5416690, 0.8333360, 0.7777810, 0.6111145, 0.6666690, 0.7083380, 0.7222240,
+	0.7777810, 0.7222240, 0.7777810, 0.7222240, 0.5833360, 0.5361130, 0.5361130,
+	0.8138910, 0.8138910, 0.2388900, 0.2666680, 0.5000020, 0.5000020, 0.5000020,
+	0.5000020, 0.5000020, 0.7375210, 0.4444460, 0.4805580, 0.7222240, 0.7777810,
+	0.5000020, 0.8611145, 0.9722260, 0.7777810, 0.2388900, 0.3194460, 0.5000020,
+	0.8333360, 0.5000020, 0.8333360, 0.7583360, 0.2777790, 0.3888900, 0.3888900,
+	0.5000020, 0.7777810, 0.2777790, 0.3333340, 0.2777790, 0.5000020, 0.5000020,
+	0.5000020, 0.5000020, 0.5000020, 0.5000020, 0.5000020, 0.5000020, 0.5000020,
+	0.5000020, 0.5000020, 0.2777790, 0.2777790, 0.3194460, 0.7777810, 0.4722240,
+	0.4722240, 0.6666690, 0.6666700, 0.6666700, 0.6388910, 0.7222260, 0.5972240,
+	0.5694475, 0.6666690, 0.7083380, 0.2777810, 0.4722240, 0.6944480, 0.5416690,
+	0.8750050, 0.7083380, 0.7361130, 0.6388910, 0.7361130, 0.6458360, 0.5555570,
+	0.6805570, 0.6875050, 0.6666700, 0.9444480, 0.6666700, 0.6666700, 0.6111130,
+	0.2888900, 0.5000020, 0.2888900, 0.5000020, 0.2777790, 0.2777790, 0.4805570,
+	0.5166680, 0.4444460, 0.5166680, 0.4444460, 0.3055570, 0.5000020, 0.5166680,
+	0.2388900, 0.2666680, 0.4888920, 0.2388900, 0.7944470, 0.5166680, 0.5000020,
+	0.5166680, 0.5166680, 0.3416690, 0.3833340, 0.3611120, 0.5166680, 0.4611130,
+	0.6833360, 0.4611130, 0.4611130, 0.4347230, 0.5000020, 1.0000030, 0.5000020,
+	0.5000020, 0.5000020 },
+    {
+	0.5805590, 0.9166720, 0.8555600, 0.6722260, 0.7333370, 0.7944490, 0.7944490,
+	0.8555600, 0.7944490, 0.8555600, 0.7944490, 0.6416700, 0.5861150, 0.5861150,
+	0.8916720, 0.8916720, 0.2555570, 0.2861130, 0.5500030, 0.5500030, 0.5500030,
+	0.5500030, 0.5500030, 0.8002530, 0.4888920, 0.5652800, 0.7944490, 0.8555600,
+	0.5500030, 0.9472275, 1.0694500, 0.8555600, 0.2555570, 0.3666690, 0.5583360,
+	0.9166720, 0.5500030, 1.0291190, 0.8305610, 0.3055570, 0.4277800, 0.4277800,
+	0.5500030, 0.8555600, 0.3055570, 0.3666690, 0.3055570, 0.5500030, 0.5500030,
+	0.5500030, 0.5500030, 0.5500030, 0.5500030, 0.5500030, 0.5500030, 0.5500030,
+	0.5500030, 0.5500030, 0.3055570, 0.3055570, 0.3666690, 0.8555600, 0.5194470,
+	0.5194470, 0.7333370, 0.7333370, 0.7333370, 0.7027820, 0.7944490, 0.6416700,
+	0.6111145, 0.7333370, 0.7944490, 0.3305570, 0.5194470, 0.7638930, 0.5805590,
+	0.9777830, 0.7944490, 0.7944490, 0.7027820, 0.7944490, 0.7027820, 0.6111145,
+	0.7333370, 0.7638930, 0.7333370, 1.0388950, 0.7333370, 0.7333370, 0.6722260,
+	0.3430580, 0.5583360, 0.3430580, 0.5500030, 0.3055570, 0.3055570, 0.5250030,
+	0.5611140, 0.4888920, 0.5611140, 0.5111140, 0.3361130, 0.5500030, 0.5611140,
+	0.2555570, 0.2861130, 0.5305590, 0.2555570, 0.8666720, 0.5611140, 0.5500030,
+	0.5611140, 0.5611140, 0.3722250, 0.4216690, 0.4041690, 0.5611140, 0.5000030,
+	0.7444490, 0.5000030, 0.5000030, 0.4763920, 0.5500030, 1.1000060, 0.5500030,
+	0.5500030, 0.550003
+    }
+};
+****************/
 
 static char* toolTip1Header[] = {
     "<script type=\"text/ecmascript\"><![CDATA[",
@@ -224,6 +391,7 @@ static char* toolTip1Header[] = {
     "       toolTip = SVGDocument.getElementById('ToolTip');",
     "       tipBox = SVGDocument.getElementById('tipbox');",
     "       tipText = SVGDocument.getElementById('tipText');",
+    "       tipText.setAttributeNS(null, 'font-family', 'Arial, sans-serif' );",
     "       tipTitle = SVGDocument.getElementById('tipTitle');",
     "       tipDesc = SVGDocument.getElementById('tipDesc');",
     "       //window.status = (TrueCoords);",
@@ -371,6 +539,7 @@ static char* toolTip2Header[] = {
     "      toolTip = SVGDocument.getElementById('ToolTip');",
     "      tipBox = SVGDocument.getElementById('tipbox');",
     "      tipText = SVGDocument.getElementById('tipText');",
+    "      tipText.setAttributeNS(null, 'font-family', 'Arial, sans-serif' );",
     "      tipTitle = SVGDocument.getElementById('tipTitle');",
     "      tipDesc1 = SVGDocument.getElementById('tipDesc1');",
     "      tipDesc2 = SVGDocument.getElementById('tipDesc2');",
@@ -783,10 +952,6 @@ static void SetFont(int face, int size, SVGDesc *ptd)
     if (lface < 1 || lface > 4) lface = 1;
     if (lsize < 1 || lsize > 24) lsize = 10;
 
-    /* original code had no units on the font-size, but I found that
-     * gave errors in the SVG renderer in FireFox under MS Windows
-     */
-    fprintf(ptd->texfp, " style=\"font-size:%dpt\" ", lsize);
     ptd->fontsize = lsize;
     ptd->fontface = lface;
 }
@@ -829,6 +994,7 @@ static void SVG_header(SVGDesc *ptd)
 	    in2dots(ptd->width), in2dots(ptd->height));
     fprintf(ptd->texfp, "     viewBox=\"0,0,%.2f,%.2f\"\n",
 	    in2dots(ptd->width), in2dots(ptd->height));
+    fprintf(ptd->texfp, "     font-family=\"Arial, Helvetica, sans-serif\"\n");
     if (ptd->toolTipMode)
 	fprintf(ptd->texfp, "     onload='Init(evt)'\n");
 
@@ -941,7 +1107,7 @@ static void SVG_NewPage(const pGEcontext gc, pDevDesc dd)
     size = ptd->fontsize;
     ptd->fontface = 0;
     ptd->fontsize = 0;
-    /*SetFont(face, size, ptd);*/
+    SetFont(gc->fontface, size, ptd);
 }
 
 /* Close down the driver */
@@ -1013,7 +1179,7 @@ static double SVG_StrWidth(const char *str,
     int size;
     double sum;
     size =  gc->cex * gc->ps + 0.5;
-    /*SetFont(font, size, ptd);*/
+    SetFont(gc->fontface, size, ptd);
     sum = 0;
     for(p=str ; *p ; p++)
 	sum += charwidth[ptd->fontface][(int)*p];
@@ -1222,7 +1388,15 @@ static void SVG_Text(double x, double y, const char *str,
     else
 	fprintf(ptd->texfp, "\" ");
 
+    if (gc->fontfamily && strlen(gc->fontfamily))
+	fprintf(ptd->texfp, "font-family=\"%s\" ", gc->fontfamily);
+    fprintf(ptd->texfp, "fill=\"%s\" ", col2RGBname(gc->col));
+
     SetFont(gc->fontface, size, ptd);
+    /* original code had no units on the font-size, but I found that
+     * gave errors in the SVG renderer in FireFox under MS Windows
+     */
+    fprintf(ptd->texfp, " style=\"font-size:%dpt\" ", ptd->fontsize);
 
     fprintf(ptd->texfp, ">");
 
@@ -1315,9 +1489,10 @@ Rboolean SVGDeviceDriver(pDevDesc dd, char *filename, char *bg, char *fg,
 
     /* Base Pointsize */
     /* Nominal Character Sizes in Pixels */
-
+    /* cra[1] tuned to get margins OK in svgplot4 in RSVGTipsDevice/man/RSVGTipsDevice.Rd 
+     * after setting default font to Arial */
     dd->cra[0] =         (6.0/12.0) * 10.0;
-    dd->cra[1] =        (10.0/12.0) * 10.0;
+    dd->cra[1] =        (13.0/12.0) * 10.0;
 
     /* Character Addressing Offsets */
     /* These offsets should center a single */
